/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * A simple FIFO cache of strings.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "scache.h"

#include "strlcpy.h"

#define CCACHE_LEN 32
#define MAX_ENTRY_LEN 512


static char scache[CCACHE_LEN][MAX_ENTRY_LEN];
static int next_scache_entry = 0;


/*
 * Initialize the scache.  Must be called before any other scache methods.
 */
void
scache_init()
{
  int i;

  for (i = 0; i < CCACHE_LEN; i++)
  {
    scache[i][0] = '\0';
  }

}


/*
 * Returns non-zero if the scache contains the specified value, zero otherwise.
 */
int
scache_contains(char *val)
{
  int i;
  int retval = 0;

  for (i = 0; i < CCACHE_LEN; i++)
  {
    if (strcmp(val, scache[i]) == 0)
    {
      retval = 1;
      break;
    }
  }

  return retval;
}


/*
 * Add a value to the scache.
 */
void
scache_add(char *val)
{
  strlcpy(scache[next_scache_entry], val, MAX_ENTRY_LEN);
  next_scache_entry = ((next_scache_entry+1) % CCACHE_LEN);
}
