/*
 *
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 *
 */

/*
 * Service Tag Client
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "sthelper.h"
#include "probe.h"
#include "stpsn.h"
#include "strlcpy.h"
#include "strlcat.h"

/*
 * Private function prototypes
 *
 */
static void display_help();
static void run_interactive();
static void print_menu();
static int  get_parm(char *, int, char *);
static int  get_line(char s[], int lim);
static void build_env_xml(char *);

char chassis_sn[128];
char sysman[128];
char cpuman[128];
char prodname[128];

int read_psn_flag = 0;

int
main(int argc, char *argv[])
{
  
	int c;
	st_t s;
	st_env_t env;
	int rc;

	int extract_flag     = 0;
	int extract_env_flag = 0;
	int extract_all_flag = 0;
	int add_flag         = 0;
	int update_flag      = 0;
	int get_flag         = 0;
	int delete_flag      = 0;
	int find_flag        = 0;
	int help_flag        = 0;
	int version_flag     = 0;
	int err_flag         = 0;
	int len_err_flag     = 0;
	extern char *optarg;
	extern int optopt;

	memset(&s,   '\0', sizeof(s));
	memset(&env, '\0', sizeof(env));
	rc = ST_ERR_NO_ERROR;

	if (argc == 1) {
		run_interactive();
		return 0;
	}

	while ((c = getopt(argc, argv, "Exaugdfhvr:p:e:P:m:A:i:t:z:S:F:I:")) != -1) {
		switch (c) {
			case 'E':
				if (add_flag || update_flag || get_flag || delete_flag ||
					find_flag || help_flag || version_flag) 
					err_flag++;
				else 
					extract_env_flag++;
				break;

			case 'x':
				if (add_flag || update_flag || get_flag || delete_flag ||
					find_flag || help_flag || version_flag) 
					err_flag++;
				else 
					extract_flag++;
				break;

			case 'a':
				if (extract_flag || update_flag || get_flag || delete_flag ||
					find_flag || help_flag || version_flag) 
					err_flag++;
				else 
					add_flag++;
				break;

			case 'u':
				if (extract_flag || add_flag || get_flag || delete_flag ||
					find_flag || help_flag || version_flag) 
					err_flag++;
				else 
					update_flag++;
				break;
		
			case 'g':
				if (extract_flag || add_flag || update_flag || delete_flag ||
					find_flag || help_flag || version_flag)
					err_flag++;
				else
					get_flag++;
				break;

			case 'd':
				if (extract_flag || add_flag || update_flag || get_flag ||
					find_flag || help_flag || version_flag)
					err_flag++;
				else
					delete_flag++;
				break;

			case 'f':
				if (extract_flag || add_flag || update_flag || get_flag ||
					help_flag || version_flag)
					err_flag++;
				else
					find_flag++;
				break;

			case 'h':
				if (extract_flag || add_flag || update_flag || get_flag ||
					find_flag || delete_flag || version_flag) 
					err_flag++;
				else 
					help_flag++;
				break;

			case 'v':
				if (extract_flag || add_flag || update_flag || get_flag ||
					find_flag || delete_flag || help_flag) 
					err_flag++;
				else 
					version_flag++;
				break;

			case 'r':
				if (strlcpy(env.root_dir, optarg, sizeof(env.root_dir)) >= sizeof(env.root_dir))
					len_err_flag++;
				 
				break;

			case 'p':
				if (strlcpy(s.product_name, optarg, sizeof(s.product_name)) >= sizeof(s.product_name))
					len_err_flag++;
				break;

			case 'e':
				if (strlcpy(s.product_version, optarg, sizeof(s.product_version)) >= sizeof(s.product_version))
					len_err_flag++;
				break;

			case 'P':
				if (strlcpy(s.product_parent, optarg, sizeof(s.product_parent)) >= sizeof(s.product_parent))
					len_err_flag++;
				break;

			case 'm':
				if (strlcpy(s.product_vendor, optarg, sizeof(s.product_vendor)) >= sizeof(s.product_vendor))
					len_err_flag++;
				break;

			case 'A':
				if (strlcpy(s.platform_arch, optarg, sizeof(s.platform_arch)) >= sizeof(s.platform_arch))
					len_err_flag++;
				break;

			case 't':
				if (strlcpy(s.product_urn, optarg, sizeof(s.product_urn)) >= sizeof(s.product_urn))
					len_err_flag++;
				break;

			case 'i':
				if (strlcpy(s.instance_urn, optarg, sizeof(s.instance_urn)) >= sizeof(s.instance_urn))
					len_err_flag++;
				break;

			case 'z':
				if (strlcpy(s.container, optarg, sizeof(s.container)) >= sizeof(s.container))
					len_err_flag++;
				break;

			case 'S':
				if (strlcpy(s.source, optarg, sizeof(s.source)) >=  sizeof(s.source))
					len_err_flag++;
				break;

			case 'F':
				if (strlcpy(s.product_parent_urn, optarg, sizeof(s.product_parent_urn)) >= 
					sizeof(s.product_parent_urn))
					len_err_flag++;
				break;

			case 'I':
				if (strlcpy(s.product_defined_inst_id, optarg, sizeof(s.product_defined_inst_id)) >=
					sizeof(s.product_defined_inst_id))
					len_err_flag++;
				break;

			case ':':   /* without operand */
				fprintf(stderr, "Option -%c requires an operand\n", optopt);
				err_flag++;
				break;

			case '?':
				fprintf(stderr, "Unrecognized option: -%c\n", optopt);
				err_flag++;
				break;
		}
	}

	if (len_err_flag) {
		printf("Max. field length has been exceeded\n");
		return -1;
	}
	if (err_flag) {
		display_help();
		return -1;
	} 

	if (extract_flag && extract_env_flag) {
	    extract_all_flag = 1;
    }

	if (version_flag) {

		/* Display version */
		printf("%s\n", SERVICE_TAG_VERSION);

	} else if (help_flag) {

		/* Display Help */
		display_help();

	} else if (extract_all_flag) {
		/* Extract XML */
		if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
			printf("%s\n", st_error(rc));
			return rc;
		}

        char env_xml[2048];
        build_env_xml(env_xml);
        printf("%s", env_xml);

        char *xml_data;
        int offset = strlen("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        rc = st_get_registry_xml(&xml_data, &env);
        printf("%s", &xml_data[offset]);
        st_free_xml(xml_data);
        st_close(&env);

        printf("</agent>\n");
    } else if (extract_env_flag) {
        /* Extract XML */
        if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
            printf("%s\n", st_error(rc));
            return rc;
        }

        char env_xml[2048];
        build_env_xml(env_xml);
        printf("%s", env_xml);
        printf("</agent>\n");

	} else if (extract_flag) {
		/* Extract XML */
		if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
			printf("%s\n", st_error(rc));
			return rc;
		}

		char *xml_data;
		rc = st_get_registry_xml(&xml_data, &env);
		printf("%s\n", xml_data);
		st_free_xml(xml_data);
		st_close(&env);

	} else if (add_flag) {

		/* Add product in service tag registry */
		if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
			printf("%s\n", st_error(rc));
			return rc;
		}
		rc = st_add(&s, &env);
		if (rc == ST_ERR_NO_ERROR) {
			printf("%s %s added\n", s.product_name, s.product_version);
			printf("Product instance URN=%s\n", s.instance_urn);
		} else {
			printf("%s\n", st_error(rc));
		}
		st_close(&env);
	} else if (update_flag) {

		/* Update product in service tag registry */
		if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
			printf("%s\n", st_error(rc));
			return rc;
		}
		rc = st_update(&s, &env);
		if (rc == ST_ERR_NO_ERROR) {
			printf("Product instance updated\n");
		} else {
			printf("%s\n", st_error(rc));
		}
		st_close(&env);
	} else if (get_flag) {
	
		/* Get service tag */
		if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
			printf("%s\n", st_error(rc));
			return rc;
		}
		rc = st_get(&s, &env);
		if (rc == ST_ERR_NO_ERROR) {
			st_debug(NULL, &s);
		} else {
			printf("%s\n", st_error(rc));
		}
		st_close(&env);
	} else if (delete_flag) {

		/* Delete service tag */
		if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
			printf("%s\n", st_error(rc));
			return rc;
		}
		rc = st_delete(s.instance_urn, &env);
		if (rc == ST_ERR_NO_ERROR) {
			printf("Product instance deleted\n");
		} else {
			printf("%s\n", st_error(rc));
		}
		st_close(&env);
	} else if (find_flag) {

		/* Find service tags by product URN */
		int i;
		int j;
		char **inst;
		if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
			printf("%s\n", st_error(rc));
			return rc;
		}
		j = st_get_instance_urns(s.product_urn, &inst, &env);
		if (j < 0) {
			rc = j;
			printf("%s\n", st_error(rc));	
		} else if (j == 0) {
			rc = ST_ERR_REC_NOT_FOUND;
			printf("%s\n", st_error(rc));	
		}
		for (i = 0; i < j; i++)
			printf("%s\n", inst[i]); 
		st_free_instance_urns(inst);
		st_close(&env);	
	} else {
		display_help();
	}
	return rc;
}


void
display_help()
{
	printf("Usage:\n");
	printf("  Interactive mode    :  no parameters\n");
	printf("  Extract ST registry :  -x [-r <dir>]\n");
	printf("  Extract Environment :  -E [-r <dir>]\n");
	printf("  Add service tag     :  -a [-i <instance URN>] -p <product> -e <product version> -t <product URN>\n");
	printf("                      :     [-F <parent URN>] -P <product parent> [-I <product defined id>] -m <product vendor>\n");
	printf("                      :     -A <arch> -z <container> -S <source> [-r <dir>]\n");
	printf("  Update service tag  :  -u -i <instance URN> -F <parent URN>  -I <product defined id> [-r <dir>]\n");
	printf("  Get service tag     :  -g -i <instance URN> [-r <dir>]\n");
	printf("  Delete service tag  :  -d -i <instance URN> [-r <dir>]\n");
	printf("  Find service tags   :  -f -t <product URN>  [-r <dir>]\n");
	printf("  Help                :  -h\n");
	printf("  Display version     :  -v\n");
}


void 
run_interactive()
{

	int      do_flag = 1;
	int      option = -1;
	int      rc;
	st_t     s;
	st_env_t env;
	char     dummy[2];
	char     *xml_data;

	memset(&env,  '\0', sizeof(env));
	printf("\n");
	print_menu();
	printf("> ");

	while ((option = getchar()) != 'q' && do_flag == 1) {
		memset(&s,    '\0', sizeof(s));

		switch (option) {
			case EOF:
				do_flag = 0;
				break;
			case 'i': 
				/* Initialize environment */
				get_line(dummy, sizeof(dummy));     /* burn the input stream char */
				get_parm(env.root_dir, sizeof(env.root_dir), "Enter root directory or <enter> for system: ");
				if ((rc = st_initialize(env.root_dir, &env)) != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
				} else {
					printf("Environment initialized\n");
				}
				printf("Press enter to continue\n");
				break;

			case 'E':
				if (st_is_initialized(&env) == ST_NOT_INITIALIZED) {
					printf("%s\n", st_error(ST_ERR_NOT_INIT));
					break;
				}
				get_line(dummy, sizeof(dummy));
				/* Display service tag registry */
                char env_xml[2048];
                build_env_xml(env_xml);
                printf("%s", env_xml);
                printf("</agent>\n");
				printf("Press enter to continue\n");
				break;
	
			case 'x': 
				if (st_is_initialized(&env) == ST_NOT_INITIALIZED) {
					printf("%s\n", st_error(ST_ERR_NOT_INIT));
					break;
				}
				get_line(dummy, sizeof(dummy));
				/* Display service tag registry */
				if ((rc = st_get_registry_xml(&xml_data, &env)) == ST_ERR_NO_ERROR) {
					printf("\n%s\n", xml_data);
				} else {
					printf("%s\n", st_error(rc));
				}
				st_free_xml(xml_data);
				printf("Press enter to continue\n");
				break;

			case 'a':
				rc = ST_ERR_NO_ERROR;
				if (st_is_initialized(&env) == ST_NOT_INITIALIZED) {
					printf("%s\n", st_error(ST_ERR_NOT_INIT));
					break;
				}
				get_line(dummy, sizeof(dummy));
				rc = get_parm(s.instance_urn, sizeof(s.instance_urn), "Enter instance URN (optional): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_name, sizeof(s.product_name), "Enter product (e.g. Sun Web Server): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_version,  sizeof(s.product_version), "Enter version (e.g. 6.1): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_urn,    sizeof(s.product_urn), "Enter product URN: ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_parent_urn, sizeof(s.product_parent_urn), "Enter product parent URN: ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_parent, sizeof(s.product_parent), "Enter product parent (e.g. JES): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_defined_inst_id, sizeof(s.product_defined_inst_id), 
					"Enter product defined instance id: ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc =  get_parm(s.product_vendor, sizeof(s.product_vendor), 
					"Enter product vendor (e.g. Sun Microsystems): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.platform_arch,  sizeof(s.platform_arch),  "Enter platform arch  (e.g. sparc): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.container,      sizeof(s.container),      "Enter container (e.g. global): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.source,         sizeof(s.source),         "Enter source (e.g. patch): ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				if ((rc = st_add(&s, &env)) == ST_ERR_NO_ERROR) {
					printf("%s %s added\n", s.product_name, s.product_version);
					printf("Product instance URN=%s\n", s.instance_urn);
				} else {
					printf("%s\n", st_error(rc));
				}
				printf("Press enter to continue\n");
				break;

			case 'u': 
				if (st_is_initialized(&env) == ST_NOT_INITIALIZED) {
					printf("%s\n", st_error(ST_ERR_NOT_INIT));
					break;
				}
				get_line(dummy, sizeof(dummy));
				rc = get_parm(s.instance_urn, sizeof(s.instance_urn), "Enter instance URN: ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_parent_urn, sizeof(s.product_parent_urn), "Enter product parent URN: ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				rc = get_parm(s.product_defined_inst_id, sizeof(s.product_defined_inst_id), 
					"Enter product defined instance id: ");
				if (rc != ST_ERR_NO_ERROR) {
					printf("%s\n", st_error(rc));
                                	break;
				}
				if ((rc = st_update(&s, &env)) == ST_ERR_NO_ERROR) {
					printf("Service tag %s updated\n", s.instance_urn);
				} else {
					printf("%s\n", st_error(rc));
				}
				printf("Press enter to continue\n");
				break;

			case 'g':
				if (st_is_initialized(&env) == ST_NOT_INITIALIZED) {
					printf("%s\n", st_error(ST_ERR_NOT_INIT));
					break;
				}
				get_line(dummy, sizeof(dummy));
				get_parm(s.instance_urn, sizeof(s.instance_urn), "Enter instance URN: ");
				if ((rc = st_get(&s, &env)) == ST_ERR_NO_ERROR) {
					st_debug(NULL, &s);
				} else {
					printf("%s\n", st_error(rc));
				}
				printf("Press enter to continue\n");
				break;

			case 'd':
				if (st_is_initialized(&env) == ST_NOT_INITIALIZED) {
					printf("%s\n", st_error(ST_ERR_NOT_INIT));
					break;
				}
				get_line(dummy, sizeof(dummy));
				get_parm(s.instance_urn, sizeof(s.instance_urn), "Enter instance URN: ");
				if ((rc = st_delete(s.instance_urn, &env)) == ST_ERR_NO_ERROR) {
					printf("Service tag %s deleted\n", s.instance_urn);
				} else {
					printf("%s\n", st_error(rc));
				}
				printf("Press enter to continue\n");
				break;

			case 'f':
				if (st_is_initialized(&env) == ST_NOT_INITIALIZED) {
					printf("%s\n", st_error(ST_ERR_NOT_INIT));
					break;
				}
				get_line(dummy, sizeof(dummy));
				get_parm(s.product_urn, sizeof(s.product_urn), "Enter product URN: ");
				int i;
				int j = 0;
				char **inst = NULL;
				j = st_get_instance_urns(s.product_urn, &inst, &env);
				if (j < 0) {
					rc  = j;
					printf("%s\n", st_error(rc));
				} else if (j == 0) {
					rc = ST_ERR_REC_NOT_FOUND;
					printf("%s\n", st_error(rc));
				}
				for (i = 0; i < j; i++)
					printf("%s\n", inst[i]);
				st_free_instance_urns(inst);
				printf("Press enter to continue\n");
				break;

			case 'h':
				printf("\n");
				print_menu();
				break;

			case '\n':
				printf("> ");
				break;

			case '\t':
			case ' ':
				break;

			default:
				printf("Unrecognized command: %c\n", option);
				break;
		}
	}
	if (st_is_initialized(&env) == ST_INITIALIZED) 
		st_close(&env);
}

void
print_menu()
{
	printf("COMMANDS:\n");
	printf("\th - Show this menu\n");
	printf("\ti - Initialize environment\n");
	printf("\tx - Extract service tag registry\n");
	printf("\tE - Extract environment data\n");
	printf("\ta - Add service tag\n");
	printf("\tu - Update service tag\n");
	printf("\tg - Get service tag\n");
	printf("\td - Delete service tag\n");
	printf("\tf - Find service tags by product URN\n");
	printf("\tq - Quit\n\n");
}


int
get_line(char s[], int lim)
{
	int c;
	int i;
	for (i=0; i < lim && (c = getchar()) != EOF && c != '\n'; i++) 
		s[i] = (char) c;
	s[i] = '\0';
	return i;
}

int
get_parm(char *data, int size, char *prompt)
{
        char line[MAX_URN_LEN];
        memset(line, '\0', sizeof(line));
        printf("%s", prompt);
        if (get_line(line, size) > 0) {
                if (strlcpy(data, line, size) >= size) {
			get_line(line, sizeof(line));  /* burn the rest of the char if len was exceeded */
			return ST_ERR_MAX_LEN_EXCEEDED;
                }
        }
	return ST_ERR_NO_ERROR;
}

char *
solaris_psn()
{
    if (read_psn_flag == 0)
    {
      read_psn_file(chassis_sn, sysman, cpuman, prodname, 128);
      read_psn_flag = 1;
    }

#ifdef SOLARIS_PSN
    if (chassis_sn == NULL || strlen(chassis_sn) == 0)
    {
      char dummy1[128];
      char dummy2[128];
      char dummy3[128];
      retrieve_solaris_system_info(chassis_sn, dummy1, dummy2, dummy3);
    }
#endif

    return chassis_sn;
}

char *
solaris_sysman()
{
    if (read_psn_flag == 0)
    {
      read_psn_file(chassis_sn, sysman, cpuman, prodname, 128);
      read_psn_flag = 1;
    }

#ifdef SOLARIS_PSN
    if (sysman == NULL || strlen(sysman) == 0)
    {
      char dummy1[128];
      char dummy2[128];
      char dummy3[128];
      retrieve_solaris_system_info(dummy1, dummy2, sysman, dummy3);
    }
#endif

    return sysman;
}

char *
solaris_cpuman()
{
    if (read_psn_flag == 0)
    {
      read_psn_file(chassis_sn, sysman, cpuman, prodname, 128);
      read_psn_flag = 1;
    }

#ifdef SOLARIS_PSN
    if (cpuman == NULL || strlen(cpuman) == 0)
    {
      char dummy1[128];
      char dummy2[128];
      char dummy3[128];
      retrieve_solaris_system_info(dummy1, cpuman, dummy2, dummy3);
    }
#endif

    return cpuman;
}

char *
solaris_prodname()
{
    if (read_psn_flag == 0)
    {
      read_psn_file(chassis_sn, sysman, cpuman, prodname, 128);
      read_psn_flag = 1;
    }

#ifdef SOLARIS_PSN
    if (prodman == NULL || strlen(prodman) == 0)
    {
      char dummy1[128];
      char dummy2[128];
      char dummy3[128];
      retrieve_solaris_system_info(dummy1, dummy2, dummy3, prodname);
    }
#endif

    return prodname;
}

static void
build_env_xml(char *agent_xml)
{
        char urn[512];
        char a_version[512];
        char r_version[512];
        char tmp[512];
        int i;
        struct probeStruct probeData;

        strlcpy(agent_xml, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n", 2048);
        strlcat(agent_xml, "<agent>\n", 2048);
        snprintf(urn, 512, "  <agent_urn>%s</agent_urn>\n", st_get_registry_urn());

        snprintf(a_version, 512,
           "  <agent_version>%s</agent_version>\n",
           SERVICE_TAG_VERSION);

        snprintf(r_version, 512,
           "  <registry_version>%s</registry_version>\n",
           SERVICE_TAG_VERSION);

        strlcat(agent_xml, urn, 2048);
        strlcat(agent_xml, a_version, 2048);
        strlcat(agent_xml, r_version, 2048);
        strlcat(agent_xml, "  <system_info>\n", 2048);

        get_probe_data(&probeData);
        for (i = 0; i < MAX_SYSCONF_FIELDS; i++)
        {
            snprintf(tmp,
                     512,
                     "    <%s>%s</%s>\n",
                     sysconfField[i],
                     probeData.sysconf[i],
                     sysconfField[i]);
            strlcat(agent_xml, tmp, 2048);
        }
        strlcat(agent_xml, "  </system_info>\n", 2048);
}
