/*
 *
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 *
 */

#pragma ident  "$Id: sysdep.c,v 1.1.1.1 2007/04/09 21:22:48 tw76360 Exp $ SMI"

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "sysdep.h"

#if defined(sun)
#include <stdlib.h>
#endif

#if defined(__linux__)
#include <time.h>
#endif

/*
 * System dependent call to get IEEE node ID.
 *
 * XXX - this is specific for Solaris and hasn't been tested on
 * other platforms.
 *
 */

void get_ieee_node_identifier(uuid_node_t *node) {

  static int inited = 0;
  static uuid_node_t saved_node;
  
  if (!inited) {

    long hostid = gethostid();

    int i;
    for (i = 0; i < 6; ++i) {
      saved_node.id[5 - i] = (byte) hostid;
      hostid = hostid >> 8;
    }

    /* because hostid isn't the mac address we have to do some funky
       stuff to meet the standard */

    saved_node.id[0] = saved_node.id[2];

    /* all MAC addresses of Sun hardware look like the 2nd and 3rd
       bytes are 00 and 20 respectivly, this needs to be validated
       and, of course, this won't work on non-Sun hardware */

    saved_node.id[1] = 0x00;
    saved_node.id[2] = 0x20;

    inited = 1;

  }
  
  *node = saved_node;

}

/*
 * System dependent call to get the current system time.  Returned as
 * 100ns ticks since Oct 15, 1582, but resolution may be less than
 * 100ns.
 */

void get_system_time(uuid_time_t *uuid_time) {

  struct timeval tp;
  gettimeofday(&tp, (struct timezone *)0);
  
  /* Offset between UUID formatted times and Unix formatted times.
     UUID UTC base time is October 15, 1582.
     Unix base time is January 1, 1970.
  */

  *uuid_time = (tp.tv_sec * 10000000) + (tp.tv_usec * 10) +
    I64(0x01B21DD213814000);

}

/*
 * System dependent call to get the node identifer used in registration.
 *
 * XXX - this is specific for Solaris and hasn't been tested on
 * other platforms.  12 is used because gethostid() returns a max 32 bit number.
 *
 */

void get_node_identifier(char *node_identifier) {
  snprintf(node_identifier, 12, "%lx", gethostid()); 
}

/*
 * System dependent call to get the node domain used in registration.
 *
 * XXX - this is specific for Solaris and hasn't been tested on
 * other platforms.
 *
 */

void get_node_domain(char *node_domain) {

#if defined(sun)
  char buf[NODE_DOMAIN_LEN];
  sysinfo(SI_SRPC_DOMAIN, buf, NODE_DOMAIN_LEN);
  snprintf(node_domain, NODE_DOMAIN_LEN, "%s", buf);

#elif defined(__linux__)
  char buf[NODE_DOMAIN_LEN];
  if (getdomainname(buf, NODE_DOMAIN_LEN) == -1)
     strcpy(buf, "unknown");
  snprintf(node_domain, NODE_DOMAIN_LEN, "%s", buf);


#endif
}

/*
 * System dependent call to get the node timezone used in registration.
 *
 * XXX - this is specific for Solaris and hasn't been tested on
 * other platforms.  128 is the max timezone buffer len in libzoneinfo.h.
 *
 */

void get_node_timezone(char *node_timezone) {
#if defined(sun)
  snprintf(node_timezone, 128, "%s", getenv("TZ"));
#elif defined(__linux__)
  snprintf(node_timezone, 128, "%s", tzname[0]);
#endif
}



