/*-------------------------------------------------*/
/* Probe Registry Header File                      */
/*-------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/utsname.h>
#include <sys/stat.h>

#define PROBE_VERSION  "1.0"
#define MAX_SYSCONF_BUF_LEN 257
#define MAX_SYSCONF_FIELDS 9
#define ONE_MB (1024 * 1024)
#define SI_SYSTEM         0
#define SI_HOST           1
#define SI_RELEASE        2
#define SI_ARCH           3
#define SI_PLATFORM       4
#define SI_MANUFACTURER   5
#define SI_CPU_MANUFACTURER 6
#define SI_SERIAL_NUMBER  7
#define SI_HOSTID         8

#define SN_SOURCE_LEN 32
#define SN_VALUE_LEN 128
#define SN_COMMAND_LEN 128

struct sn_info {
  int return_code;
  char sn_source[SN_SOURCE_LEN];
  char sn_value[SN_VALUE_LEN];
  char command[SN_COMMAND_LEN];
};

/*-------------------------------------------------*/
/* Structures                                      */
/*-------------------------------------------------*/
struct probeStruct {
  char sysconf[MAX_SYSCONF_FIELDS][257];
};

/*-------------------------------------------------*/
/* Function Prototypes (Public)                    */
/*-------------------------------------------------*/
void get_probe_data(struct probeStruct *);
int getNumProcs();
void read_psn_file(char *, char *, char *, char *, int);

/*-------------------------------------------------*/
/* Function Prototypes (Private)                   */
/*-------------------------------------------------*/
static int init(int);

/*-------------------------------------------------*/
/* Global variables                                */
/*-------------------------------------------------*/

static char *sysconfField[MAX_SYSCONF_FIELDS] =
   {"system", "host", "release",
    "architecture", "platform", "manufacturer",
    "cpu_manufacturer", "serial_number", "hostid"};

/* cpubuffer needs to be large enough to read in the entire /proc/cpuinfo file */
char cpubuffer[4*1024];

/* membuffer needs to be large enough to read in the entire /proc/meminfo file */
char membuffer[4*1024];
